/*  --------------------------------------------------------------------------------------
    Program  : PlotHistoN09.sas

    Author   :  Perry Watts
    Date     :  Sunday, May 24, 2009

    Path     :  c:\Your Path

    Purpose  :  Create Histograms with:
                a) Endpoints rather than Midpoints
                b) Accommodate EMF or CGM graphics devices
                c) Bar segments that represent subgroups
                d) Accommodate Normal and Gamma probability distribution functions.
                e) Accommodate Kernal Density Function.

    Parms    :  Name             Description                                      Default
                ---------------  -----------------------------------------------  --------
                inds             input data set
                grfDevice        So far: EMF or CGMOF97L                          CGMOF97L
                EMFbgColor       Background color for EMF to match ppt slides
                                  (CGMOF97L is transparent)
                grfFile          Graphics file name with PATH
                BarColor         Single color for a bar.  Replaced with           grayCC
                                  colorFmt when subGroupVar is NOT blank 
                mpLabelYvN       Label midpoints (YvN)                            N
                mpTextColor      Color for MP text labels when mpLabelYvN=Y       black
                subGroupVar      subgroup variable for multiple colored bars.
                                  If blank, then subgroups are not being 
                                  accommodated.                              
                colorFmt         for coloring bar segments that correspond to
                                  frequencies of subGroupVar. Start values are
                                  ALWAYS ordinal numbers ranging from 1 to n
                                  (the number of subgroups).                    
                sortByVorF       Sort Segments by V (variable Value i.e.          V
                                  ASCII COLLATING SEQUENCE) or F(Freq)
                legendFmt        Legend Format for subgroup. Start values are
                                  ALWAYS ordinal numbers ranging from 1 to n
                                  (the number of subgroups). If BLANK, no legend
                                  is produced.
                xLgnd            X coordinate (percent of data area) for Legend   2
                xvar             variable plotted on midpoint axis
                xmin             XMIN (such that data MIN is calculated) or       XMIN
                                  a value
                xmax             XMAX (such that data MAX is evaluated) or        XMAX
                                  a value
                xdataOffset      For HistoConfig=1,3:                             0
                                   #Units by which to decrease AND increase         
                                   underlying X-Axis range
                                 For HistoConfig=2:
                                   Offset in pct assigned in Axis stmt
                HistoConfig      1=Number of Bars                                 1
                                 2=BY
                                 3=Intermediate cutpoints(for uneven bars)
                ConfigInfo       If 1, then actual number of bars desired
                                 If 2, Interval is supplied
                                 If 3, values demarcated by spaces are for
                                       inner cutpoints.
                Yorigin                                                           12
                pctSize          In Percent (to match annotate)                    5
                XaxisLbl         X-Axis-Label  
                XvalFmt          Display format for X-axis                        best. 
                YaxisLbl         Y-Axis-Label (If Percent, Pct, or %,             Frequency
                                  and subgroup is turned on, then Percents
                                  are plotted).     
                Yby              By value for Y-axis
                CurveType        Normal Curve(N), Gamma Curve(G), KDF(K)          blank
                                  or NO curve (blank)
                Title1           Title1 text can include optional MOVE commands
                                  for better centering with the CGMOF97L device 
                Title2           Title2 text (often used for total counts)

     Macros    
     invoked :  %UnevenTicksAxis, %MkUnderlyingScale, %getAxisMin, %getAxisMax, 
                %InFcn, and %getIncr.
    -----------------------------------------------------------------------------------  */

 %macro PlotHistoN09(inds=, grfDevice=CGMOF97L, EMFbgColor=, grfFile=, barColor=grayCC, mpTextColor=black,
   subGroupVar=, colorFm=, sortByVorF=V, legendFmt=, xlgnd=2, xvar=, xmin=XMIN, xmax=XMAX, xdataOffset=0,
   HistoConfig=1, ConfigInfo=, yorigin=12, pctSize=3, XaxisLbl=, xValFmt=best., YaxisLbl=Frequency,
   yby=, mpLabelYvN=N, curveType=, title1=, title2=);

 %let YaxisLbl=%nrbquote(&yaxisLbl);
 %let XaxisLbl=%nrbquote(&xaxisLbl);

 %annomac;
 symbol1; symbol2; symbol3; pattern1; axis1; axis2; footnote1;
 %let titleSize = %sysevalf(&pctSize + 0.5);
 filename GSFOut "&grfFile";
 goptions reset=all;

  /* ACCOMMODATE EITHER CGMOF97L OR EMF GRAPHICS DEVICES */
 %if %upcase(&grfDevice) eq CGMOF97L %then %do;
    %let ftext= HWCGM001; %let ftitle=HWCGM002;
 %end;
 %else %if %upcase(&grfDevice) eq EMF %then %do;
    %let ftext="Arial"; %let ftitle="Arial/Bold";
    %if &EMFbgColor ne %then %do;
        goptions cback=&EMFbgColor;
    %end;
 %end;
 %else %do;
    %put !!ERROR: Need to define FONT macro variables for device &grfDevice;
    %put !! Reassign to device CGMOF97L;
    %let grfDevice=CGMOF97L; %let ftext= HWCGM001; %let ftitle=HWCGM002;
 %end;

 goptions device=&grfDevice ctext=black htitle=&titlesize.pct htext=&pctSize.pct display 
   rotate=landscape gunit=pct polygonfill fill
   ftext=&ftext ftitle=&ftitle gsfname=GSFOut gsfmode=replace;

/* GET DATA MINS AND MAXES IF NECESSARY - FOR XFMT CONSTRUCTION */
%if %upcase(&xMIN) eq XMIN OR %upcase(&xMAX) eq XMAX %then %do;
/*get min and max values*/
  proc univariate data=&inds noprint;
    var &xvar;
    output out=minmax min=xmin max=xmax;
  run;
/*create macro variables*/
  data _null_;
    set minmax;
   %if &xMin eq XMIN %then call symput('xmin',put(xmin,4.2)); ;
   %if &xMax eq XMAX %then call symput('xmax',put(xmax,4.2)); ;
  run;
%end;

%if &histoConfig eq 2 %then %do;
  %let xby=&configInfo;
  %let xMin=%getAxisMin(Min=&xmin, by=&xby);
  %let xMax=%getAxisMax(Max=&xmax, by=&xby);
%end;

/***************************************************************
  build a format from xmin, xmax, [number of bars OR Interval]
  -- i.e. get values for:
     ncut     (# cut points)
     interval (between cut points)
     cut&i    (value of the ith cutpoint)
  example of proc format for 3 even width bars:
    value xfmt 0.25-<1.5='0.25' 1.5-<2.75='1.5' 2.75 - 4.0='2.75';
****************************************************************/
%if &histoConfig ne 3 %then %do;
  %if &histoConfig eq 1 %then %do;
    %let nbars=&configInfo;
    %let ncut=%eval(&nbars-1);
    %let interval = %sysevalf((&xmax - &xmin)/&nbars);
  %end;
  %else %do;
    %let nbars=%sysevalf((&xmax - &xmin)/&xby);
    %let ncut=%eval(&nbars-1);
    %let interval=&xby;
  %end;
  %let start=&xmin;
  %do i=1 %to &ncut;
     %let cut&i = %sysevalf(&start + &interval);
     %let start =&&cut&i;
  %end;
%end;
%else %do; /*have enumerated cutpoints*/
 %let start=&xmin;
 %let i=1;
 %do %while (%scan(&ConfigInfo,&i,%str( )) ne %str());
    %let start = %scan(&ConfigInfo,&i,%str( ));
    %let cut&i = &start;
    %let i=%eval(&i+1);
 %end;
 %let ncut=%eval(&i-1);
%end;
  proc format;
    value xfmt &xmin -< &cut1 = "&xmin"
      %let ncut_1=%eval(&ncut-1);
      %do i= 1 %to &ncut_1;
         %let iplus1 = %eval(&i+1);
         &&cut&i -< &&cut&iplus1 = "&&cut&i"
      %end;
      &&cut&ncut - &xmax = "&&cut&ncut"
    ;
  run;

/* CREATE INPUT DATA FOR THE UNEVENTICKSAXIS MACRO */
  proc format library=WORK
    cntlout=XaxisTicks(keep=start end);
    select xfmt;
  run;
  data XaxisTicks(keep=xtick);
    set XaxisTicks;
    xtick=input(left(start),best.); output;
    xtick=input(left(end),best.); output;
  run;

  /* FONT AND FONTBOLD PARMS IN UNEVENTICSAXIS REQUIRE SINGLE-QUOTES INSTEAD OF THE
     GOPTIONS DOUBLE-QUOTES */ 
  %if %index(&ftext,%str(%")) gt 0 %then %do;
   %let ftext = %sysfunc(translate(&ftext,%str(%'),%str(%")));
   %let ftitle = %sysfunc(translate(&ftitle,%str(%'),%str(%")));
  %end;

/* ALSO WORKS WITH EVEN INTERVALS BY SIMULATING A LIST RATHER THAN RANGE FORMAT*/
%UnevenTicksAxis(inDS=xAxisTicks, xvar=xtick, pctSize=&pctSize, xlabel=&XaxisLbl, 
                 yOrigin=&yOrigin, xvalfmt=&xvalfmt., font=&ftext, fontbold=&ftitle)


/* SUBGROUP PROCESSING BY FREQUENCY OR PERCENT - IF REQUESTED -- TO LINE 416 */
%if &subgroupVar ne %then %do;
   %if %InFcn(&YaxisLbl , %nrbquote(pct percent %)) %then %do;
     %let yAxVar=rowPercent;
     %let yaxisLbl=Stacked Group Percent;
   %end;
   %else %do;
     %let yAxVar=frequency;
   %end;

 %let rc=%sysfunc(open(&inds));
 %let vnum=%sysfunc(varnum(&rc,%upcase(&subgroupVar)));
 %let vtype=%sysfunc(vartype(&rc,&vnum));
 %let rc=%sysfunc(close(&rc));

  proc sql noprint;
    create table SG(keep=&subgroupVar) as
    select distinct &subgroupVar
    from &inds
    order by &subgroupVar;
    select count(*) into :nSG from SG;
  quit;
 %let nSG=%left(&nSG);
 %if &vtype eq C %then %do;
  data cntlinInFm(keep=fmtname type start label);
    retain type 'I' fmtname 'sgFmt';
    length start $30;
    set sg;
    start=&subgroupVar;
    label=put(_n_,1.);
  run; 
  proc format cntlin=cntlinInFm;
  run;
 %end;
 %else %do;
  data cntlinFm(keep=fmtname type start label);
    retain type 'N' fmtname 'sgNFmt';
    set sg;
    start=&subgroupVar;
    label=put(_n_,1.);
  run; 
  proc format cntlin=cntlinFm;
  run;
 %end;

 /*** CREATE PLOT DATA FOR SUBGROUPS BY FIRST BINNING THE RAW DATA VIA FORMAT ***/
 /* FORMAT X COORDINATES */
  data inds(keep=xx &xvar sgOrd);
    set &inds;
    xx=input(put(&xvar,xfmt.),best.);
    %if &Vtype eq C %then %do;
       sgOrd=input(&subgroupVar,sgFmt.); 
    %end;
    %else %do; 
       sgOrd=input(put(&subgroupVar,sgNfmt.),2.); 
    %end;
  run;
  proc sort data=inds;
    by &xvar;
  run;
  
 /* NEED ZEROES FOR MERGE WITH SUMMARY DATA BELOW -- FOR ZERO HEIGHT BINS. */
 proc format library=work
  cntlout=xfmtOUT(keep=start);
  select xfmt;
 run;

 /* ZEROES BY SGORD AND XX */
 data zeros_sg(keep=xx sgord frequency RowPercent);
  set xfmtOUT;
  do sgord=1 to &nSG;
    xx=input(left(start),best.);
    frequency=0; RowPercent=0;
    output;
  end;
 run;
   
 /* ZEROES BY XX ONLY */
 data zeros(keep=xx frequency RowPercent);
  set zeros_sg;
  by xx;
  if first.xx;
 run;

/* GET XX||SG FREQUENCIES AND PERCENTS FOR RESPONSE AXIS */
 ODS listing close;
 ods output clear;
 ods output CrossTabFreqs = CTfreqouts;
    proc freq data=inds;
      tables sgord*xx;
    run;
 ods listing;
 ods output close;

 data xxSGds1(keep=xx sgord frequency RowPercent);
   set CTfreqOuts;
   if _TYPE_ eq '11';
 run;
 proc sort data=xxSGds1;
  by xx sgOrd;
 run;
 data xxSGds1;
   merge zeros_sg xxSGds1;
   by xx sgOrd;
 run;
 proc sql noprint;
   create table xxSGds as 
     select * from xxSGds1
  /* SORT BY V(ALUE) (ASCII) OR F(REQUENCY) */
     %if %upcase(&SortByVorF) eq F %then %do;
      order by xx, frequency descending ;
    %end;
    %else %do; 
      order by xx, sgOrd ;
    %end;
 quit;

 proc sql noprint;
   create table xxDS as
   select xx, sum(frequency) as frequency, sum(rowPercent)as rowPercent
   from xxSGds
   group by xx
   order by xx;
 quit;
 data xxDS;
   merge zeros xxds;
   by xx;
 run;

/* CREATE DATA SET TO BE PLOTTED IN GPLOT. 
   YAXVAR(Y-AXIS VARIABLE)= SAS-DEFINED VARIABLES FROM PROC FREQ: 
   FREQUENCY OR ROWPERCENT. */
  data pltds(keep=xx yy grp);
    retain grp 1;
    set xxDS end=last;
    lagy=lag(&yAxVar);
    if _n_ = 1 then do;
      yy = 0; output;
    end;
    else do;
      yy = lagy; output;
      yy = 0; output;
    end;
    if last then do;
      yy = &yAxVar; xx = &xmax; output;
      yy = 0; output;
    end;
  run;

/* CREATE DATA SET FOR MULTI-COLORED BARS - 2 passes required */
  proc sql noprint;
   create table nodups as
     select distinct xtick
   from xAxisTicks;

   select distinct xtick into :arrayVals separated by ' '
   from nodups;
   select count(xtick) into :nticks
   from nodups;
  quit;

  /* PASS #1 */
  data sgBars(keep=xxLeft xxRight sgOrd cumFreq);
    array cutpoints[&nticks.] _temporary_ (&arrayVals);
    retain iindex 1;
    retain cumfreq;
    set xxSGds;
    by XX;
    if first.XX then do;
      iindex+1;
      cumfreq=&yAxVar;
    end;
    else cumfreq=cumfreq+&yAxVar;
    xxLeft=XX;
    xxRight=cutpoints[iindex];
  run;

  /* PASS #2 */
 data BarFill(keep=xx1 yy1 xx2 yy2 sgOrd);
    set sgBars end=last;
    by xxLeft;
    lagy=lag(cumfreq);
    if first.xxleft then do;
      xx1=xxleft; yy1 = 0; xx2=xxright; yy2=cumfreq; output;
    end;
    else do;
      xx1=xxleft; yy1=lagy; xx2=xxright; yy2=cumfreq; output;
    end;
  run;

  data annoBarFill;
    retain xsys '2' ysys '2' hsys '3';
    length function color style $8;
    set BarFill; by xx1;
    color=put(sgOrd,&colorFm.);
    function='move'; x=xx1; y=yy1; output;
    function='bar'; x=xx2; y=yy2; line=3; style='solid'; output;
  run;

  /* CREATE A LEGEND IF PARAMETER LEGENDFMT IS NOT NULL */
  %if &LegendFmt ne %then %do;
   %let xplus3=%sysevalf(&xLgnd + 3);
   data annoL(keep=function text xsys ysys hsys x y size position color when);
    length function color $8 text $30;
    retain xsys '1' ysys '1' hsys'3' when 'a' position '6'; 
    %let y=100;
    %do i = 1 %to &nsg;
      %let y=%sysevalf(&y - 7);
      function='symbol'; text='dot'; x=&xLgnd; y=&y; size=4.5;
         color="black"; output;
      function='symbol'; text='dot';  size=3.5;
         color="%sysfunc(putn(&i,&colorFm.))"; output;
      function='label'; color='black'; x=&xplus3; y=%sysevalf(&y +1); size=&pctSize; 
         text="%sysfunc(putn(&i,&LegendFmt.))"; output;
    %end;
  run;
  data annoAxisX;
    set annoAxisX annoL;
  run;
  %end;
%end; /* --------------------SUBGROUP PROCESSING --------------------------- */

%else %do; /*SINGLE BAR COLOR (NO SUBGROUPS AND NO PERCENT) */
  %let YAxVar=Frequency;

 /* PROVIDE FOR ZERO HEIGHT BINS */
 proc format library=work
  cntlout=xfmtOUT(keep=start);
  select xfmt;
 run; 
 data zeros(keep=xx frequency);
  set xfmtOut;
  xx=input(left(start),best.);
  frequency=0;
 run;

/*** CREATE PLOT DATA BY FIRST BINNING THE RAW DATA VIA FORMAT ***/
/* FORMAT X COORDINATES */
  data inds(keep=xx &xvar);
    set &inds;
    xx=input(put(%left(&xvar),xfmt.),best.);
  run;
  proc sort data=inds;
    by &xvar;
  run;

/* GET FREQUENCIES FOR RESPONSE AXIS.  RENAME IS REQUIRED FOR JOINT PROCESSING
   OF SUBGROUP AND NO-GROUP PROCESSING THAT STARTS AT LINE 482 */
  proc summary data=inds nway;
    class xx;
    output out=xxDS(rename=(_freq_= frequency));
  run;
  data xxDS;
    merge zeros xxDS;
    by xx;
  run;

/* CREATE DATA SET TO BE PLOTTED IN GPLOT */
  data pltds(keep=xx yy grp);
    retain grp 1;
    set xxDS end=last;
    lagy=lag(frequency);
    if _n_ = 1 then do;
      yy = 0; output;
    end;
    else do;
      yy = lagy; output;
      yy = 0; output;
    end;
    if last then do;
      yy = frequency; xx = &xmax; output;
      yy = 0; output;
    end;
  run;

/* DERIVE ANNOTATE BARS FROM PLTDS */  
  data annoBarFill;
    %dclanno;
    %system(2,2,3);
    set pltds;
    xx1=lag(xx); yy1=lag(yy); xx2=xx; yy2=yy;
    if xx1 ne . AND yy1 eq 0;
    %bar(xx1,yy1,xx2,yy2,&barColor,0,solid);
  run;
%end; /* -------------------------SINGLE BAR COLOR -- NO SUBGROUPS -------------- */

/* IF DESIRED, LABEL HISTO MIDPOINTS WITH FREQS OR PCTS -- FOR 0 -> MANY SUBGROUPS */
%if &mpLabelYvN eq Y %then %do;
 /* GET MIDPOINT X-COORDINATE WITH CONTROL-OUT */
  proc format library=WORK
   cntlout=mp(keep=start end label);
   select xfmt;
  run;
  data mp(keep=low high midpoint);
   set mp;
   low=input(left(start),best.);
   high=input(left(end),best.);
   midpoint=low + ((high-low)/2.0);
  run;

/* CREATE MPFMT WITH CONTROL-IN */
  data cntlinMP(keep=fmtname type start label);
    retain type 'N' fmtname 'mpFmt';
    length label $8;
    set mp;
    start=low;
    label=put(midpoint,best8.);
  run;   
  proc format library=WORK cntlin=cntlinMP;
  run;

/* ASSIGN VARIABLE MP WITH MPFMT */
  data xxDS;
    set xxDS;
    mp=input(put(xx,mpfmt.),best.);
  run;

/* CREATE ANNOMP DATA SET */
  proc format;
    picture pctRfm (round) low-high='009.9%';
  run;
  data annoMP;
  %dclanno;
    length text chtext $30; /* %dclanno takes care of the rest */
  %system(2,2,3);
    set xxDS;
    %if %upcase(&yAxVar) eq FREQUENCY %then %do;
       chtext=left(put(&yAxVar,best.));
    %end;
    %else %do;
       chtext=left(put(&yAxVar,pctRfm.));
    %end;
    %label(mp,&yAxVar, chtext, &mpTextColor, 0, 0, &pctSize, &ftext, 2);
  run;

  data annoText;
    set annoAxisX annoMP;
  run;
%end;

/* (UNDERLYING) HORIZONTAL AXIS IS THE SAME REGARDLESS OF THE PARAMETER SETTINGS 
   FOR CURVES (DENSITY FUNCTIONS) - BECAUSE DENSITY FUNCTIONS ARE TRUNCATED AT
   HISTOGRAM MIN AND MAX VALUES. */
 axis2 label=none
       %if %upcase(&grfDevice) eq EMF %then w=1; %else w=5; 
       value=none major=none minor=none
       origin=(,&yorigin.) 
       %if &histoConfig eq 2 %then 
        order=(&xmin to &xmax by &ConfigInfo) offset=(&xdataOffset.pct,);
       %else 
        offset=(0pct,) 
        order=(%MkUnderlyingScale(calcXMin=&xMin, calcXMax=&XMax, OffSet=&xdataOffset));
 ;
  title1 move=(+10pct,+0pct) "&title1" ; 

  %if %length(&title2.) gt 3 %then %do;
    %if %upcase(&yaxVar) eq ROWPERCENT %then %do;
      %let totPct=%sysevalf(&nsg * 100);
      %let title2= %trim(&title2 = &totPct. %);
    %end;
    %if %upcase(&grfdevice) eq CGMOF97L %then %do;
      title2 h=0.75 " ";
      title3 move=(+10pct,+0pct) "&title2";
    %end;
    %else %if %upcase(&grfdevice) eq EMF %then %do;
      title2 move=(+10pct,+0pct) "&title2";
    %end; 
  %end;

  /* OPTIONAL CURVE GENERATION PROCESSING */

  /* CHECK PARAMETER VALUE*/
 %if &curveType ne %then %do;
   %if %InFcn(&curveType , %str(G Gamma)) %then %let curveType=G;
   %else %if %InFcn(&curveType , %str(N Normal Norm)) %then %let curveType=N;
   %else %if %InFcn(&curveType , %str(K Kernel KDF)) %then %let curveType=K;
   %else %do;
     put "!! The value for macro parameter curveType set to &curveType is in ERROR";
     put "curveType reset to blank";
     %let curveType= ;
   %end;
 %end;

  /* NO CURVE PROCESSING */
 %if &curveType eq %then %do;
/* GRAPHICS COMMANDS: */
  %if %upcase(&grfDevice) eq EMF %then %do;
    symbol1 V=none C=black L=1 W=1.5 I=steprj;
  %end;
  %else %do;
    symbol1 V=none C=black L=1 W=5 I=steprj;
  %end;

/* GET YMAX FOR VERTICAL AXIS */
  proc sql noprint;
    select max(yy) into :Ymax from pltDS;
  quit;
  %let yMax=%getAxisMax(Max=&ymax, by=&yby);
  %if &grfDevice eq CGMOF97L %then %do;
   axis1 label=(A=90 font=&ftitle "&YaxisLbl") major=(w=5 h=0.75) minor=(n=1 w=5 h=0.5)
       order=(0 to &Ymax by &yby) offset=(0,1) w=5 length=6.5in; 
  %end;
  %else %do;
   axis1 label=(A=90 font=&ftitle "&YaxisLbl") minor=(n=1) order=(0 to &Ymax by &yby) offset=(0,1) w=1;
 %end;
   proc gplot data=pltDS %if &mpLabelYvN eq N %then anno=annoAxisX; %else anno=annoText; ;
    plot yy*xx /vaxis=axis1
                haxis=axis2
                noframe
                anno=annoBarFill;
   run; quit;
 %end; /* NO CURVE PROCESSING */

/* *****
   TO PLOT THE NORMAL OR GAMMA CURVE: 
   See CDF functions (cumulative distribution funtions) on p. 421 and
   425 in the 9.13 Language Dictionary.  See PDF functions (probability density functions) on p. 700 
   and 703 in the 9.13 Language Dictionary.
   TO PLOT KDF CURVE:
   See PROC KDE in the STAT manual
   WHEN CURVES ARE PLOTTED BY MACRO THEN Y-MAX CAN BE ACCURATELY CALCULATED. 
***** */
/* REQUESTING A LEGITIMATE CURVE -- TO END OF MACRO (LINE 737)*/
 %else %do; 
  /* OBTAIN TOTAL AREA OF HISTOGRAM FROM ANNOBARFILL */  
  data _null_;
    retain area 0;
    set annoBarFill end=last;
    by xx1;
    if last.xx1;
    area=area + (xx2 - xx1) * yy2;
    if last then call symput('HistoArea',put(area,best.));
  run;
  %if %upcase(&curveType) eq K %then %do;
    %let binwidth=%sysevalf((&XMAX -&XMIN)/400);
  /* KDE DEFAULTS: BWM=1 (BANDWIDTH MULTIPLIER), METHOD=SNR (SIMPLE NORMAL REFERENCE),
     NGRID=4O1 (401-1 = 400 FOR DIVISOR ABOVE)*/
     proc kde data=&inDS;
       univar &xvar /gridl=&xmin gridu=&xmax out=o;
     run;
     data o;
       set o;
       areaKDEbin=&binwidth * density;
     run;
     proc sql noprint;
       select sum(areaKDEbin) into :subrange from o;
     quit;
     data ProbCurve;
       retain grp 2;
       set o;
       xx=value;
       yy=density * (&histoArea/&subrange);
     run;
  %end;
  %else %do; /* NORMAL OR GAMMA CURVE */
   proc sql noprint;
     select mean(&xvar), std(&xvar), var(&xvar) into :mu, :sigma, :sigmaSquared
     from &inds;
   quit;
   %let Xrange = %sysevalf(&XMax - &Xmin);
   %let By = %getIncr(range=&Xrange);

   %if %upcase(&curveType) eq N %then %do;
     %let FofX0 = %sysfunc(CDF(NORMAL,&xmin.,&mu.,&sigma.));
     %let FofXn = %sysfunc(CDF(NORMAL,&xmax.,&mu.,&sigma.));
     %let subRange=%sysevalf(&FofXn - &FofX0);
   %end;
   %else %if %upcase(&curveType) eq G %then %do;
     %let Lambda = %sysevalf(&sigmaSquared./&mu);
     %let alpha = %sysevalf(&mu./&Lambda);
     %let FofX0 = %sysfunc(CDF(GAMMA,&xmin.,&alpha.,&lambda.));
     %let FofXn = %sysfunc(CDF(GAMMA,&xmax.,&alpha.,&lambda.));
     %let subRange=%sysevalf(&FofXn - &FofX0);
   %end;

   data ProbCurve;
     retain grp 2;
     do xx= &xmin to &xmax by &by;
     %if %upcase(&curveType) eq N %then %do;          
       ProbY=PDF('NORMAL',xx,&mu.,&sigma.);
     %end;
     %else %do;
       ProbY=PDF('GAMMA',xx,&alpha.,&lambda.);
     %end;
       YY = (&HistoArea * ProbY) / &subrange; 
       output;
     end;
     if xx ne &xmax then do; /*AT THE END OF THE DO-LOOP - MAY HAVE UNDER-SHOT*/
       xx=&xmax;
       %if %upcase(&curveType) eq N %then %do;          
         ProbY=PDF('NORMAL',xx,&mu.,&sigma.);
       %end;
       %else %do;
         ProbY=PDF('GAMMA',xx,&alpha.,&lambda.);
       %end;
       YY = (&HistoArea * ProbY) / &subrange;
       output;
     end;
   run;
  %end;

  data pltDS(keep=xx yy grp);
    set pltds probCurve;
  run;

/* GRAPHICS COMMANDS: */
  %if %upcase(&grfDevice) eq EMF %then %do;
    symbol1 V=none C=black L=1 W=1.5 I=steprj;
    symbol2 V=none C=black L=1 W=2 I=join;
  %end;
  %else %do;
    symbol1 V=none C=black L=1 W=3 I=steprj;
    symbol2 V=none C=black L=1 W=10 I=join;
  %end;

/* GET YMAX (ADJUSTED FOR THE NORMAL CURVE) FOR VERTICAL AXIS */
  proc sql noprint;
    select max(yy) into :Ymax from pltDS;
  quit;
  %let yMax=%getAxisMax(Max=&ymax, by=&yby);

  /* DEFINE THE VERTICAL AXIS*/
  %if &grfDevice eq CGMOF97L %then %do;
   axis1 label=(A=90 font=&ftitle "&YaxisLbl") major=(w=5 h=0.75) minor=(n=1 w=5 h=0.5)
       order=(0 to &Ymax by &yby) offset=(0,1) w=5 length=6.5in; 
  %end;
  %else %do;
   axis1 label=(A=90 font=&ftitle "&YaxisLbl") minor=(n=1)
       order=(0 to &Ymax by &yby) offset=(0,1) w=1;
  %end;
  /* PLOT THE HISTOGRAM */
   proc gplot data=pltDS %if &mpLabelYvN eq N %then anno=annoAxisX; %else anno=annoText; ;
     plot yy*xx=grp /vaxis=axis1
                   haxis=axis2
                   noframe nolegend
                   anno=annoBarFill;
   run; quit;
 %end; /*PLOT HISTOGRAM WITH A DENSITY FUNCTION CURVE */
 %mend plotHistoN09;
