/*  --------------------------------------------------------------------------------------
    Program  : PlotHistoSummaryDat.sas

    Author   :  Perry Watts
    Date     :  Wednesday, April 23, 2008

    Path     :  C:\N09\Histograms\MacSummary

    Purpose  :  Create Histograms with endpoints rather than midpoints

    Parms    :  Name             Description                                      Default
                ---------------  -----------------------------------------------  --------
                inds             input data set
                grfDevice        So far: EMF or CGMOF97L                          CGMOF97L
                grfFile          Graphics file name with PATH
                xvar             variable plotted on the X axis
                yvar             variable plotted on the Y axis
                xdataOffset      #Units by which to decrease AND increase          0                                         
                                   underlying X-Axis range
                Yorigin          In Percent.                                      12
                pctSize          In Percent (to match annotate)                    3
                XaxisLbl         X-Axis-Label  
                XvalFmt          Display format for X-axis                        best. 
                YaxisLbl         Y-Axis-Label                                     Frequency
                Yby              By value for Y-axis
                ListFreqsYvN     Display frequencies at bar midpoints (YvN)       N
                binColor         Color of bins
                title1           title1 text including optional move commands.
                                 Text is enclosed in quotes.
     Macros    
     invoked :  %UnevenTicksAxis and %MkUnderlyingHistoScale.
    -----------------------------------------------------------------------------------  */


 %macro PlotHistoSummaryDat(inds=, grfDevice=CGMOF97L, grfFile=, xvar=, yvar=, xdataOffset=0, 
                            yorigin=12, pctSize=3, XaxisLbl=, xValFmt=best., 
                            YaxisLbl=Frequency, yby=, ListFreqsYvN=N, binColor=, title1=);

 %annomac;
 symbol1; symbol2; symbol3; pattern1; axis1; axis2; footnote1;
 %let titleSize = %sysevalf(&pctSize + 0.5);
 filename GSFOut "&grfFile";
 goptions reset=all;

 %if %upcase(&grfDevice) eq CGMOF97L %then %do;
    %let ftext= HWCGM001; %let ftitle=HWCGM002;
 %end;
 %else %if %upcase(&grfDevice) eq EMF %then %do;
    %let ftext="Arial"; %let ftitle="Arial/Bold";
 %end;
 %else %do;
    %put !!ERROR: Need to define FONT macro variables for device &grfDevice;
    %put !! Reassign to device CGMOF97L;
    %let grfDevice=CGMOF97L; %let ftext= HWCGM001; %let ftitle=HWCGM002;
 %end;

 goptions device=&grfDevice ctext=black htitle=&titlesize.pct htext=&pctSize.pct display 
   rotate=landscape gunit=pct polygonfill fill
   ftext=&ftext ftitle=&ftitle gsfname=GSFOut gsfmode=replace;

 data inds;
   set &inds end=last;
   lagYvar=lag(&yvar);
   if _n_ eq 1 then call symput('xmin',put(&xvar,best.));
   if last then call symput('xmax',put(&xvar,best.));
   xvar=&xvar;
   yvar=&yvar;
   if yvar eq . then yvar=lagYvar;
 run;
   
 %if %index(%upcase(&ftext),ARIAL) gt 0 %then %do;
   %let ftext='ARIAL'; %let ftitle='ARIAL/BOLD';
 %end;

/* ALSO WORKS WITH EVEN INTERVALS BY SIMULATING A LIST RATHER THAN RANGE FORMAT*/
%UnevenTicksAxis(inDS=inDS, xvar=xvar, pctSize=&pctSize, xlabel=&XaxisLbl,
                 yOrigin=&yOrigin, xvalfmt=&xvalfmt., font=&ftext, fontbold=&ftitle)

/* CREATE DATA SET TO BE PLOTTED IN GPLOT */
  data pltds(keep=xx yy);
    set inds end=last;
    xx=xvar;
    lagy=lag(yvar);
    if _n_ = 1 then do;
      yy = 0; output;
    end;
    else do;
      yy = lagy; output;
      yy = 0; output;
    end;
   run;

/* DERIVE ANNOTATE BARS FROM PLTDS */  
  data annoBarFill;
    %dclanno;
    %system(2,2,3);
    set pltds;
    when='B';
    xx1=lag(xx); yy1=lag(yy); xx2=xx; yy2=yy;
    if xx1 ne . AND yy1 eq 0;
    %bar(xx1,yy1,xx2,yy2,&binColor,0,solid);
  run;

/* IF DESIRED, LABEL HISTO MIDPOINTS WITH FREQUENCIES */
%if &ListFreqsYvN eq Y %then %do;
  data mp(keep=low high midpoint yy2);
    set annoBarFill;
    low=xx1;
    high=xx2;
    midpoint=low + ((high-low)/2.0);
  run;
 
/* CREATE ANNOMP DATA SET */
  data annoMP;
  %dclanno;
    length text $30; /* %dclanno takes care of the rest */
  %system(2,2,3);
    set mp;
    text=left(put(yy2,best.));
    %label(midpoint,yy2, text, black, 0, 0, &pctSize, &ftext, 2);
  run;
  data annoText;
    set annoAxisX annoMP;
  run;
%end;

/* GRAPHICS COMMANDS: */
  symbol1 V=none C=black L=1 W=1 I=steprj;

/* VERTICAL AXIS */
  proc sql noprint;
    select max(yy) into :Ymax from pltDS;
  quit;
  %let yMax=%getAxisMax(Max=&ymax, by=&yby);

 axis1 label=(A=90 font=&ftitle "&YaxisLbl") 
  minor=(n=1 h=0.5) major=(h=0.75)
  order=(0 to &Ymax by &yby) offset=(0,1) w=1 
  %if &grfDevice eq CGMOF97L %then length=6.5in; ;
/* (UNDERLYING) HORIZONTAL AXIS */
 axis2 label=none w=1 value=none major=none minor=none
  origin=(,&yorigin.) offset=(0pct,) 
   order=(%MkUnderlyingScale(calcXMin=&xMin, calcXMax=&XMax, OffSet=&xdataOffset));

  title1 &title1 ;
  title3 " ";

  /* PLOT THE HISTOGRAM */
  proc gplot data=pltds %if &ListFreqsYvN eq N %then anno=annoAxisX; %else anno=annoText; ;
    plot yy*xx /vaxis=axis1
                haxis=axis2
                noframe
                anno=annoBarFill;
  run; quit;
 %mend plotHistoSummaryDat;
