/*  ---------------------------------------------------------------------------
    Program  :  UnevenTicksAxis.sas

    Path     :  C:\Your Path

    Purpose  :  Create an uneven X axis where ticks coincide with a finite
                number of X-coordinates in the data. Ticks are labeled with
                values for the X-coordinates. Defaults accommodate Device=CGMOF97L.

    Parms    :  Name                   Description                         Required
                ---------          --------------------------------        --------
                INDS               Input data set name                     yes
                XVAR               Horizontal axis variable name           yes
                PCTSIZE            Text Size in PCT                        yes
                XLABEL             X-axis label                            yes
                YORIGIN            Vertical origin in percent              yes
                XVALFMT            Format used to display tick             no
                                    marks along the X-Axis. 
                TICKLENGTH         Length of Ticks                         no
                TICKWIDTH          Width of Ticks                          no
                FONT               e.g. Hwcgm001 (default), 'Arial'        yes 
                FONTBOLD           e.g. Hwcgm002 (default), 'Arial/Bold'   yes
    Input    :  &inDS
    Output   :  work.annoAxisX
    ------------------------------------------------------------------------  */

  %macro UnevenTicksAxis(inDS=, xvar=, pctSize=, xlabel=, yOrigin=, XvalFmt=, 
                         tickLength=, tickWidth=, font=HWcgm001, fontBold=HWcgm002);
     %annomac;
     %local textSeparation labelYcoord;
     %if %upcase(&font) eq HWCGM001 %then %do;
        %if &ticklength eq %then %do; /* FITTED FOR PCTSIZE=3 */
          %let ticklength=1.25;  %let tickWidth=0.11;
        %end;
        %let textSeparation=%sysevalf(&tickLength * 2.75);
        %let labelYcoord=%sysevalf(&textSeparation * 1.75);
     %end;
     %else %do; /* So far the only other font is ARIAL */
        %if &ticklength eq %then %do; /* FITTED FOR PCTSIZE=5 */
          %let ticklength=1.5;  %let tickWidth=0.2;
        %end;
       %let textSeparation=%sysevalf(1.5 * &tickLength);
        %let labelYcoord=%sysevalf(&textSeparation * 3);
     %end;  

     proc sql noprint;
       create table distinctXtick as
       select distinct &xvar as xtick from &inDS
       order by &xvar;
     quit;
     data distinctXtick;
       length DisplayX $15;
       set distinctXtick;
       %if &Xvalfmt eq %then %do;
         fmtYvN=0;
         DisplayX=left(put(xtick,best.));
       %end;
       %else %do;
         fmtYvN=1;
         displayX=left(trim(put(xtick, &xValFmt.)));
       %end;
     run;
     data annoAxisX;
       retain xsys '2' hsys '3' when 'A';
       length text $30 function color $8 style $15;
       set distinctXtick end=last;
       function='move'; ysys='1'; x=xtick; y=0; output;
       function='draw'; ysys='7'; y=-&tickLength; 
         color='black';
         line=1; size=&tickWidth; output;
       function='move'; y=-&textSeparation; output;
       function='cntl2txt'; output;
       function='label'; call missing(x,y); position='5'; text = displayX; 
          size=&pctSize; style="&font"; output; 
       if last then do;   
          function='move'; xsys='1'; x=50; y=-&LabelYcoord; output;
          function='cntl2txt'; output;
          function='label'; call missing(x,y); position='5'; text = "&xLabel"; 
            size=&pctSize; style="&fontBold"; output; 
       end;
     run;
 %mend UnevenTicksAxis;
